@@ -23,6 +23,7 @@
 #include "progress.h"
 #include "RLPainter.h"
 #include "smalldialogs.h"
+#include "StandardMenus.h"
 #include "Utils.h"
 // Standard c/c++ includes
 #include <memory>
@@ -71,8 +72,6 @@ bool KDiff3App::m_bTripleDiff = false;
 std::unique_ptr<Options> gOptions = std::make_unique<Options>();
 
 boost::signals2::signal<QString(), FirstNonEmpty<QString>> KDiff3App::getSelection;
-boost::signals2::signal<bool(), or_> KDiff3App::allowCopy;
-boost::signals2::signal<bool(), or_> KDiff3App::allowCut;
 
 /*
     To be a constexpr the QLatin1String constructor must be given the size of the string explicitly.
@@ -113,13 +112,28 @@ bool KDiff3App::isDirComparison() const
 /*
     Don't call completeInit from here it will be called in KDiff3Shell as needed.
 */
-KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Shell* pKDiff3Shell):
+KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Shell* pKDiff3Shell, const FileNames& names):
     QMainWindow(pParent)
 {
     setWindowFlags(Qt::Widget);
     setObjectName(name);
     m_pKDiff3Shell = pKDiff3Shell;
 
+    //Get SourceData objects intalized as soon as possiable or wierd errors can happen on startup.
+    if(!names.fn1.isEmpty())
+    {
+        m_sd1->setFilename(names.fn1);
+        m_bDirCompare = m_sd1->isDir();
+    }
+    if(!names.fn2.isEmpty())
+    {
+        m_sd2->setFilename(names.fn2);
+    }
+    if(!names.fn3.isEmpty())
+    {
+        m_sd3->setFilename(names.fn3);
+    }
+
     m_pCentralWidget = new QWidget(this);
     QVBoxLayout* pCentralLayout = new QVBoxLayout(m_pCentralWidget);
     pCentralLayout->setContentsMargins(0, 0, 0, 0);
@@ -155,7 +169,7 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Shell* pKDiff3
     m_pOptionDialog->readOptions(KSharedConfig::openConfig());
 
     // Option handling.
-    QtSizeType argCount = KDiff3Shell::getParser()->optionNames().count() + KDiff3Shell::getParser()->positionalArguments().count();
+    qsizetype argCount = KDiff3Shell::getParser()->optionNames().count() + KDiff3Shell::getParser()->positionalArguments().count();
     bool hasArgs = argCount > 0;
     if(hasArgs)
     {
@@ -298,14 +312,24 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Shell* pKDiff3
     }
     g_pProgressDialog->setStayHidden(m_bAutoMode);
 
+    //initView does first time setup for ui.
+    initView();
     ///////////////////////////////////////////////////////////////////
-    // call inits to invoke all other construction parts
-    // Warning: Call this before connecting KDiff3App::slotUpdateAvailabilities or
+    //
+    // Warning: Call initActions before connecting KDiff3App::slotUpdateAvailabilities or
     //  calling KXMLGUIClient::setXMLFile or KXMLGUIClient::createGUI
     initActions(actionCollection());
+    m_pDirectoryMergeWindow->initDirectoryMergeActions(this, actionCollection());
 
     initStatusBar();
 
+    autoAdvance->setEnabled(true);
+    sourceMask(0, 0);
+
+    showWindowA->setChecked(true);
+    showWindowB->setChecked(true);
+    showWindowC->setChecked(true);
+
     m_pFindDialog = new FindDialog(this);
     chk_connect_a(m_pFindDialog, &FindDialog::findNext, this, &KDiff3App::slotEditFindNext);
 
@@ -331,7 +355,25 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Shell* pKDiff3
     }
 
     slotRefresh();
+    //Connect singals that may depend on KDiff3App being setup.
+    m_pMergeResultWindow->setupConnections(this);
+    m_pDirectoryMergeWindow->setupConnections(this);
 
+    chk_connect(QApplication::clipboard(), &QClipboard::dataChanged, this, [this]() { stdMenus->StandardMenus::slotClipboardChanged(); });
+    chk_connect_q(this, &KDiff3App::sigRecalcWordWrap, this, &KDiff3App::slotRecalcWordWrap);
+    chk_connect_a(this, &KDiff3App::finishDrop, this, &KDiff3App::slotFinishDrop);
+
+    connections.push_back(StandardMenus::allowSave.connect(boost::bind(&KDiff3App::canSave, this)));
+    connections.push_back(StandardMenus::allowSaveAs.connect(boost::bind(&KDiff3App::canSaveAs, this)));
+    connections.push_back(StandardMenus::allowCut.connect(boost::bind(&KDiff3App::canCut, this)));
+    connections.push_back(StandardMenus::allowCopy.connect(boost::bind(&KDiff3App::canCopy, this)));
+
+    chk_connect_a(qApp, &QApplication::focusChanged, this, &KDiff3App::slotFocusChanged);
+    chk_connect_a(this, &KDiff3App::updateAvailabilities, this, &KDiff3App::slotUpdateAvailabilities);
+}
+
+void KDiff3App::initView()
+{
     m_pDirectoryMergeDock = new QDockWidget(i18n("Directory merge"), this);
     m_pDirectoryMergeWindow = new DirectoryMergeWindow(m_pDirectoryMergeDock, *this);
     m_pDirectoryMergeDock->setObjectName("DirectoryMergeDock");
@@ -343,26 +385,140 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Shell* pKDiff3
     m_pDirectoryMergeInfoDock->setWidget(m_pDirectoryMergeInfo);
     m_pDirectoryMergeInfoDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
     m_pDirectoryMergeWindow->setDirectoryMergeInfo(m_pDirectoryMergeInfo);
-    //Warning: Make sure DirectoryMergeWindow::initActions is called before this point or we can crash when selectionChanged is sent.
-    m_pDirectoryMergeWindow->setupConnections(this);
+
     addDockWidget(Qt::LeftDockWidgetArea, m_pDirectoryMergeDock);
     splitDockWidget(m_pDirectoryMergeDock, m_pDirectoryMergeInfoDock, Qt::Vertical);
 
-    chk_connect_a(QApplication::clipboard(), &QClipboard::dataChanged, this, &KDiff3App::slotClipboardChanged);
-    chk_connect_q(this, &KDiff3App::sigRecalcWordWrap, this, &KDiff3App::slotRecalcWordWrap);
-    chk_connect_a(this, &KDiff3App::finishDrop, this, &KDiff3App::slotFinishDrop);
+    //m_pMainWidget // Contains vertical splitter and horiz scrollbar
+    QVBoxLayout* pVLayout = new QVBoxLayout(m_pMainWidget);
+    pVLayout->setContentsMargins(0, 0, 0, 0);
+    pVLayout->setSpacing(0);
 
-    connections.push_back(allowCut.connect(boost::bind(&KDiff3App::canCut, this)));
-    connections.push_back(allowCopy.connect(boost::bind(&KDiff3App::canCopy, this)));
+    QSplitter* pVSplitter = new QSplitter();
+    pVSplitter->setObjectName("VSplitter");
+    pVSplitter->setOpaqueResize(false);
+    pVSplitter->setOrientation(Qt::Vertical);
+    pVLayout->addWidget(pVSplitter);
 
-    m_pDirectoryMergeWindow->initDirectoryMergeActions(this, actionCollection());
+    QWidget* pDiffWindowFrame = new QWidget(); // Contains diff windows, overview and vert scrollbar
+    pDiffWindowFrame->setObjectName("DiffWindowFrame");
+    QHBoxLayout* pDiffHLayout = new QHBoxLayout(pDiffWindowFrame);
+    pDiffHLayout->setContentsMargins(0, 0, 0, 0);
+    pDiffHLayout->setSpacing(0);
+    pVSplitter->addWidget(pDiffWindowFrame);
 
-    if(qApp != nullptr)
-        chk_connect_a(qApp, &QApplication::focusChanged, this, &KDiff3App::slotFocusChanged);
+    m_pDiffWindowSplitter = new QSplitter();
+    m_pDiffWindowSplitter->setObjectName("DiffWindowSplitter");
+    m_pDiffWindowSplitter->setOpaqueResize(false);
 
-    chk_connect_a(this, &KDiff3App::updateAvailabilities, this, &KDiff3App::slotUpdateAvailabilities);
+    m_pDiffWindowSplitter->setOrientation(gOptions->m_bHorizDiffWindowSplitting ? Qt::Horizontal : Qt::Vertical);
+    pDiffHLayout->addWidget(m_pDiffWindowSplitter);
+
+    m_pOverview = new Overview();
+    m_pOverview->setObjectName("Overview");
+    pDiffHLayout->addWidget(m_pOverview);
+
+    DiffTextWindow::mVScrollBar = new QScrollBar(Qt::Vertical, pDiffWindowFrame);
+    pDiffHLayout->addWidget(DiffTextWindow::mVScrollBar);
+
+    chk_connect_a(m_pOverview, &Overview::setLine, DiffTextWindow::mVScrollBar, &QScrollBar::setValue);
+    chk_connect_a(this, &KDiff3App::showWhiteSpaceToggled, m_pOverview, &Overview::slotRedraw);
+    chk_connect_a(this, &KDiff3App::changeOverViewMode, m_pOverview, &Overview::setOverviewMode);
+
+    m_pDiffTextWindowFrame1 = new DiffTextWindowFrame(m_pDiffWindowSplitter, e_SrcSelector::A, m_sd1, *this);
+    m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame1);
+    m_pDiffTextWindowFrame2 = new DiffTextWindowFrame(m_pDiffWindowSplitter, e_SrcSelector::B, m_sd2, *this);
+    m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame2);
+    m_pDiffTextWindowFrame3 = new DiffTextWindowFrame(m_pDiffWindowSplitter, e_SrcSelector::C, m_sd3, *this);
+    m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame3);
+    m_pDiffTextWindow1 = m_pDiffTextWindowFrame1->getDiffTextWindow();
+    m_pDiffTextWindow2 = m_pDiffTextWindowFrame2->getDiffTextWindow();
+    m_pDiffTextWindow3 = m_pDiffTextWindowFrame3->getDiffTextWindow();
+
+    m_pDiffTextWindowFrame1->setupConnections(this);
+    m_pDiffTextWindowFrame2->setupConnections(this);
+    m_pDiffTextWindowFrame3->setupConnections(this);
+
+    // Merge window
+    m_pMergeWindowFrame = new QWidget(pVSplitter);
+    m_pMergeWindowFrame->setObjectName("MergeWindowFrame");
+    pVSplitter->addWidget(m_pMergeWindowFrame);
+    QHBoxLayout* pMergeHLayout = new QHBoxLayout(m_pMergeWindowFrame);
+    pMergeHLayout->setContentsMargins(0, 0, 0, 0);
+    pMergeHLayout->setSpacing(0);
+    QVBoxLayout* pMergeVLayout = new QVBoxLayout();
+    pMergeHLayout->addLayout(pMergeVLayout, 1);
+
+    m_pMergeResultWindowTitle = new WindowTitleWidget();
+    pMergeVLayout->addWidget(m_pMergeResultWindowTitle);
+
+    m_pMergeResultWindow = new MergeResultWindow(m_pMergeWindowFrame, statusBar());
+    pMergeVLayout->addWidget(m_pMergeResultWindow, 1);
+
+    MergeResultWindow::mVScrollBar = new QScrollBar(Qt::Vertical, m_pMergeWindowFrame);
+    pMergeHLayout->addWidget(MergeResultWindow::mVScrollBar);
+
+    QList<qint32> sizes = pVSplitter->sizes();
+    qint32 total = sizes[0] + sizes[1];
+    if(total < 10)
+        total = 100;
+    sizes[0] = total / 2;
+    sizes[1] = total / 2;
+    pVSplitter->setSizes(sizes);
+
+    QList<qint32> hSizes = {1, 1, 1};
+
+    m_pDiffWindowSplitter->setSizes(hSizes);
+
+    m_pMergeResultWindow->installEventFilter(m_pMergeResultWindowTitle); // for focus tracking
+
+    QHBoxLayout* pHScrollBarLayout = new QHBoxLayout();
+    pVLayout->addLayout(pHScrollBarLayout);
+    m_pHScrollBar = new ReversibleScrollBar(Qt::Horizontal, &gOptions->m_bRightToLeftLanguage);
+    pHScrollBarLayout->addWidget(m_pHScrollBar);
+    m_pCornerWidget = new QWidget(m_pMainWidget);
+    pHScrollBarLayout->addWidget(m_pCornerWidget);
+
+    chk_connect_a(DiffTextWindow::mVScrollBar, &QScrollBar::valueChanged, m_pOverview, &Overview::setFirstLine);
+    chk_connect_a(DiffTextWindow::mVScrollBar, &QScrollBar::valueChanged, m_pDiffTextWindow1, &DiffTextWindow::setFirstLine);
+    chk_connect_a(m_pHScrollBar, &ReversibleScrollBar::valueChanged2, m_pDiffTextWindow1, &DiffTextWindow::setHorizScrollOffset);
+    m_pDiffTextWindow1->setupConnections(this);
+
+    chk_connect_a(DiffTextWindow::mVScrollBar, &QScrollBar::valueChanged, m_pDiffTextWindow2, &DiffTextWindow::setFirstLine);
+    chk_connect_a(m_pHScrollBar, &ReversibleScrollBar::valueChanged2, m_pDiffTextWindow2, &DiffTextWindow::setHorizScrollOffset);
+    m_pDiffTextWindow2->setupConnections(this);
+
+    chk_connect_a(DiffTextWindow::mVScrollBar, &QScrollBar::valueChanged, m_pDiffTextWindow3, &DiffTextWindow::setFirstLine);
+    chk_connect_a(m_pHScrollBar, &ReversibleScrollBar::valueChanged2, m_pDiffTextWindow3, &DiffTextWindow::setHorizScrollOffset);
+    m_pDiffTextWindow3->setupConnections(this);
+
+    MergeResultWindow* p = m_pMergeResultWindow;
+    chk_connect_a(MergeResultWindow::mVScrollBar, &QScrollBar::valueChanged, p, &MergeResultWindow::setFirstLine);
+
+    chk_connect_a(m_pHScrollBar, &ReversibleScrollBar::valueChanged2, p, &MergeResultWindow::setHorizScrollOffset);
+    chk_connect_a(p, &MergeResultWindow::modifiedChanged, m_pMergeResultWindowTitle, &WindowTitleWidget::slotSetModified);
+
+    chk_connect_a(p, &MergeResultWindow::setFastSelectorRange, m_pDiffTextWindow1, &DiffTextWindow::setFastSelectorRange);
+    chk_connect_a(p, &MergeResultWindow::setFastSelectorRange, m_pDiffTextWindow2, &DiffTextWindow::setFastSelectorRange);
+    chk_connect_a(p, &MergeResultWindow::setFastSelectorRange, m_pDiffTextWindow3, &DiffTextWindow::setFastSelectorRange);
+    chk_connect_a(m_pDiffTextWindow1, &DiffTextWindow::setFastSelectorLine, p, &MergeResultWindow::slotSetFastSelectorLine);
+    chk_connect_a(m_pDiffTextWindow2, &DiffTextWindow::setFastSelectorLine, p, &MergeResultWindow::slotSetFastSelectorLine);
+    chk_connect_a(m_pDiffTextWindow3, &DiffTextWindow::setFastSelectorLine, p, &MergeResultWindow::slotSetFastSelectorLine);
+    chk_connect_a(m_pDiffTextWindow1, &DiffTextWindow::gotFocus, p, &MergeResultWindow::updateSourceMask);
+    chk_connect_a(m_pDiffTextWindow2, &DiffTextWindow::gotFocus, p, &MergeResultWindow::updateSourceMask);
+    chk_connect_a(m_pDiffTextWindow3, &DiffTextWindow::gotFocus, p, &MergeResultWindow::updateSourceMask);
+    chk_connect_a(m_pDirectoryMergeInfo, &DirectoryMergeInfo::gotFocus, p, &MergeResultWindow::updateSourceMask);
+
+    chk_connect_a(m_pDiffTextWindow1, &DiffTextWindow::resizeHeightChangedSignal, this, &KDiff3App::resizeDiffTextWindowHeight);
+    // The following connects cause the wordwrap to be recalced thrice, just to make sure. Better than forgetting one.
+    chk_connect_a(m_pDiffTextWindow1, &DiffTextWindow::resizeWidthChangedSignal, this, &KDiff3App::postRecalcWordWrap);
+    chk_connect_a(m_pDiffTextWindow2, &DiffTextWindow::resizeWidthChangedSignal, this, &KDiff3App::postRecalcWordWrap);
+    chk_connect_a(m_pDiffTextWindow3, &DiffTextWindow::resizeWidthChangedSignal, this, &KDiff3App::postRecalcWordWrap);
+
+    m_pDiffTextWindow1->setFocus();
+    m_pMainWidget->setMinimumSize(50, 50);
+    m_pCornerWidget->setFixedSize(DiffTextWindow::mVScrollBar->width(), m_pHScrollBar->height());
 }
-
 /*
     This function is only concerned with qt objects that don't support canCut.
     allowCut() or's the results from all canCut signals
@@ -376,6 +532,20 @@ bool KDiff3App::canCut()
     return (qobject_cast<QLineEdit*>(focus) != nullptr || qobject_cast<QTextEdit*>(focus) != nullptr);
 }
 
+/*canSave*/
+bool KDiff3App::canSave()
+{
+    bool bSavable = m_pMergeWindowFrame->isVisible() && m_pMergeResultWindow->getNumberOfUnsolvedConflicts() == 0;
+
+    return bSavable;
+}
+
+/*canSaveAs*/
+bool KDiff3App::canSaveAs()
+{
+    return (m_bOutputModified && StandardMenus::allowSave());
+}
+
 /*
     Please do not add logic for MergeResultWindow or DiffTextWindow here they have their own handlers.
     This function is only concerned with qt objects that don't support canCopy.
@@ -440,25 +610,11 @@ void KDiff3App::doFileCompare()
     mainInit(m_totalDiffStatus);
 }
 
-void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QString& fn3)
+void KDiff3App::completeInit()
 {
     bool openError = false;
     bool bSuccess = true;
 
-    if(!fn1.isEmpty())
-    {
-        m_sd1->setFilename(fn1);
-        m_bDirCompare = m_sd1->isDir();
-    }
-    if(!fn2.isEmpty())
-    {
-        m_sd2->setFilename(fn2);
-    }
-    if(!fn3.isEmpty())
-    {
-        m_sd3->setFilename(fn3);
-    }
-
     //Should not fail ever.
     assert(m_bDirCompare == m_sd1->isDir());
     if(m_bDirCompare != m_sd2->isDir() || (!m_sd3->isEmpty() && m_bDirCompare != m_sd3->isDir()))
@@ -481,8 +637,6 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
         showMainWindow();
 
     g_pProgressDialog->setStayHidden(false);
-    //initView does first time setup for ui.
-    initView();
 
     if(bSuccess)
     {
@@ -552,7 +706,7 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
     if(statusBar() != nullptr)
         statusBar()->setSizeGripEnabled(true);
 
-    slotClipboardChanged(); // For initialisation.
+    stdMenus->slotClipboardChanged(); // For initialisation.
 
     Q_EMIT updateAvailabilities();
 
@@ -609,42 +763,6 @@ void KDiff3App::initActions(KActionCollection* ac)
         KMessageBox::error(nullptr, "actionCollection==0");
         exit(-1); //we cannot recover from this.
     }
-    fileOpen = KStandardAction::open(this, &KDiff3App::slotFileOpen, ac);
-    fileOpen->setStatusTip(i18n("Opens documents for comparison..."));
-
-    fileReload = GuiUtils::createAction<QAction>(i18n("Reload"), QKeySequence(QKeySequence::Refresh), this, &KDiff3App::slotReload, ac, u8"file_reload");
-
-    fileSave = KStandardAction::save(this, &KDiff3App::slotFileSave, ac);
-    fileSave->setStatusTip(i18n("Saves the merge result. All conflicts must be solved!"));
-    fileSaveAs = KStandardAction::saveAs(this, &KDiff3App::slotFileSaveAs, ac);
-    fileSaveAs->setStatusTip(i18n("Saves the current document as..."));
-#ifndef QT_NO_PRINTER
-    filePrint = KStandardAction::print(this, &KDiff3App::slotFilePrint, ac);
-    filePrint->setStatusTip(i18n("Print the differences"));
-#endif
-    fileQuit = KStandardAction::quit(this, &KDiff3App::slotFileQuit, ac);
-    fileQuit->setStatusTip(i18n("Quits the application"));
-
-    editUndo = KStandardAction::undo(this, &KDiff3App::slotEditUndo, ac);
-    editUndo->setShortcuts(QKeySequence::Undo);
-    editUndo->setStatusTip(i18n("Undo last action."));
-
-    editCut = KStandardAction::cut(this, &KDiff3App::slotEditCut, ac);
-    editCut->setShortcuts(QKeySequence::Cut);
-    editCut->setStatusTip(i18n("Cuts the selected section and puts it to the clipboard"));
-    editCopy = KStandardAction::copy(this, &KDiff3App::slotEditCopy, ac);
-    editCopy->setShortcut(QKeySequence::Copy);
-    editCopy->setStatusTip(i18n("Copies the selected section to the clipboard"));
-    editPaste = KStandardAction::paste(this, &KDiff3App::slotEditPaste, ac);
-    editPaste->setStatusTip(i18n("Pastes the clipboard contents to current position"));
-    editPaste->setShortcut(QKeySequence::Paste);
-    editSelectAll = KStandardAction::selectAll(this, &KDiff3App::slotEditSelectAll, ac);
-    editSelectAll->setStatusTip(i18n("Select everything in current window"));
-    editFind = KStandardAction::find(this, &KDiff3App::slotEditFind, ac);
-    editFind->setShortcut(QKeySequence::Find);
-    editFind->setStatusTip(i18n("Search for a string"));
-    editFindNext = KStandardAction::findNext(this, &KDiff3App::slotEditFindNext, ac);
-    editFindNext->setStatusTip(i18n("Search again for the string"));
 
     viewStatusBar = KStandardAction::showStatusbar(this, &KDiff3App::slotViewStatusBar, ac);
     viewStatusBar->setStatusTip(i18n("Enables/disables the statusbar"));
@@ -652,6 +770,13 @@ void KDiff3App::initActions(KActionCollection* ac)
     QAction* pAction = KStandardAction::preferences(this, &KDiff3App::slotConfigure, ac);
     pAction->setText(i18n("Configure KDiff3..."));
 
+    stdMenus->setup(this, ac);
+
+    editFind = KStandardAction::find(this, &KDiff3App::slotEditFind, ac);
+    editFind->setShortcut(QKeySequence::Find);
+    editFind->setStatusTip(i18n("Search for a string"));
+    editFindNext = KStandardAction::findNext(this, &KDiff3App::slotEditFindNext, ac);
+    editFindNext->setStatusTip(i18n("Search again for the string"));
 #include "xpm/autoadvance.xpm"
 #include "xpm/currentpos.xpm"
 #include "xpm/down1arrow.xpm"
@@ -670,31 +795,31 @@ void KDiff3App::initActions(KActionCollection* ac)
 #include "xpm/up2arrow.xpm"
 #include "xpm/upend.xpm"
 
-    mGoCurrent = GuiUtils::createAction<QAction>(i18n("Go to Current Delta"), QIcon(QPixmap(currentpos)), i18n("Current\nDelta"), QKeySequence(Qt::CTRL | Qt::Key_Space), this, &KDiff3App::slotGoCurrent, ac, "go_current");
+    mGoCurrent = GuiUtils::createAction<QAction>(i18n("Go to Current Delta"), QIcon(QPixmap(currentpos)), i18n("Current\nDelta"), QKeySequence(Qt::CTRL | Qt::Key_Space), this, [this] { Q_EMIT goCurrent(); }, ac, "go_current");
 
-    mGoTop = GuiUtils::createAction<QAction>(i18n("Go to First Delta"), QIcon(QPixmap(upend)), i18n("First\nDelta"), this, &KDiff3App::slotGoTop, ac, "go_top");
+    mGoTop = GuiUtils::createAction<QAction>(i18n("Go to First Delta"), QIcon(QPixmap(upend)), i18n("First\nDelta"), this, [this] { Q_EMIT goTop(); }, ac, "go_top");
 
-    mGoBottom = GuiUtils::createAction<QAction>(i18n("Go to Last Delta"), QIcon(QPixmap(downend)), i18n("Last\nDelta"), this, &KDiff3App::slotGoBottom, ac, "go_bottom");
+    mGoBottom = GuiUtils::createAction<QAction>(i18n("Go to Last Delta"), QIcon(QPixmap(downend)), i18n("Last\nDelta"), this, [this] { Q_EMIT goBottom(); }, ac, "go_bottom");
 
     QString omitsWhitespace = ".\n" + i18nc("Tooltip explanation text", "(Skips white space differences when \"Show White Space\" is disabled.)");
     QString includeWhitespace = ".\n" + i18nc("Tooltip explanation text", "(Does not skip white space differences even when \"Show White Space\" is disabled.)");
-    mGoPrevDelta = GuiUtils::createAction<QAction>(i18n("Go to Previous Delta"), QIcon(QPixmap(up1arrow)), i18n("Prev\nDelta"), QKeySequence(Qt::CTRL | Qt::Key_Up), this, &KDiff3App::slotGoPrevDelta, ac, "go_prev_delta");
+    mGoPrevDelta = GuiUtils::createAction<QAction>(i18n("Go to Previous Delta"), QIcon(QPixmap(up1arrow)), i18n("Prev\nDelta"), QKeySequence(Qt::CTRL | Qt::Key_Up), this, [this] { Q_EMIT goPrevDelta(); }, ac, "go_prev_delta");
     mGoPrevDelta->setToolTip(mGoPrevDelta->text() + omitsWhitespace);
-    mGoNextDelta = GuiUtils::createAction<QAction>(i18n("Go to Next Delta"), QIcon(QPixmap(down1arrow)), i18n("Next\nDelta"), QKeySequence(Qt::CTRL | Qt::Key_Down), this, &KDiff3App::slotGoNextDelta, ac, "go_next_delta");
+    mGoNextDelta = GuiUtils::createAction<QAction>(i18n("Go to Next Delta"), QIcon(QPixmap(down1arrow)), i18n("Next\nDelta"), QKeySequence(Qt::CTRL | Qt::Key_Down), this, [this] { Q_EMIT goNextDelta(); }, ac, "go_next_delta");
     mGoNextDelta->setToolTip(mGoNextDelta->text() + omitsWhitespace);
-    mGoPrevConflict = GuiUtils::createAction<QAction>(i18n("Go to Previous Conflict"), QIcon(QPixmap(up2arrow)), i18n("Prev\nConflict"), QKeySequence(Qt::CTRL | Qt::Key_PageUp), this, &KDiff3App::slotGoPrevConflict, ac, "go_prev_conflict");
+    mGoPrevConflict = GuiUtils::createAction<QAction>(i18n("Go to Previous Conflict"), QIcon(QPixmap(up2arrow)), i18n("Prev\nConflict"), QKeySequence(Qt::CTRL | Qt::Key_PageUp), this, [this] { Q_EMIT goPrevConflict(); }, ac, "go_prev_conflict");
     mGoPrevConflict->setToolTip(mGoPrevConflict->text() + omitsWhitespace);
     mGoNextConflict = GuiUtils::createAction<QAction>(i18n("Go to Next Conflict"), QIcon(QPixmap(down2arrow)), i18n("Next\nConflict"), QKeySequence(Qt::CTRL | Qt::Key_PageDown), this, &KDiff3App::slotGoNextConflict, ac, "go_next_conflict");
     mGoNextConflict->setToolTip(mGoNextConflict->text() + omitsWhitespace);
-    mGoPrevUnsolvedConflict = GuiUtils::createAction<QAction>(i18n("Go to Previous Unsolved Conflict"), QIcon(QPixmap(prevunsolved)), i18n("Prev\nUnsolved"), QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_PageUp), this, &KDiff3App::slotGoPrevUnsolvedConflict, ac, "go_prev_unsolved_conflict");
+    mGoPrevUnsolvedConflict = GuiUtils::createAction<QAction>(i18n("Go to Previous Unsolved Conflict"), QIcon(QPixmap(prevunsolved)), i18n("Prev\nUnsolved"), QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_PageUp), this, [this] { Q_EMIT goPrevUnsolvedConflict(); }, ac, "go_prev_unsolved_conflict");
     mGoPrevUnsolvedConflict->setToolTip(mGoPrevUnsolvedConflict->text() + includeWhitespace);
     mGoNextUnsolvedConflict = GuiUtils::createAction<QAction>(i18n("Go to Next Unsolved Conflict"), QIcon(QPixmap(nextunsolved)), i18n("Next\nUnsolved"), QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_PageDown), this, &KDiff3App::slotGoNextUnsolvedConflict, ac, "go_next_unsolved_conflict");
     mGoNextUnsolvedConflict->setToolTip(mGoNextUnsolvedConflict->text() + includeWhitespace);
     mGotoLine = GuiUtils::createAction<QAction>(i18nc("Title for menu item", "Go to Line"), QIcon(QPixmap(gotoline)), i18nc("Text used for toolbar button.", "Go\nLine"), QKeySequence(Qt::CTRL | Qt::Key_G), this, &KDiff3App::slotGoToLine, ac, "go_to_line");
     mGotoLine->setToolTip(mGoNextUnsolvedConflict->text() + ".\n" + i18nc("Tooltip Text", "Goto specified line."));
-    chooseA = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Select Line(s) From A"), QIcon(QPixmap(iconA)), i18nc("Text used for select A toolbar button.", "Choose\nA"), QKeySequence(Qt::CTRL | Qt::Key_1), this, &KDiff3App::slotChooseA, ac, "merge_choose_a");
-    chooseB = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Select Line(s) From B"), QIcon(QPixmap(iconB)), i18nc("Text used for select B when toolbar button.", "Choose\nB"), QKeySequence(Qt::CTRL | Qt::Key_2), this, &KDiff3App::slotChooseB, ac, "merge_choose_b");
-    chooseC = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Select Line(s) From C"), QIcon(QPixmap(iconC)), i18nc("Text used for select C toolbar button.", "Choose\nC"), QKeySequence(Qt::CTRL | Qt::Key_3), this, &KDiff3App::slotChooseC, ac, "merge_choose_c");
+    chooseA = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Select Line(s) From A"), QIcon(QPixmap(iconA)), i18nc("Text used for select A toolbar button.", "Choose\nA"), QKeySequence(Qt::CTRL | Qt::Key_1), this, [this]{choose(e_SrcSelector::A);}, ac, "merge_choose_a");
+    chooseB = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Select Line(s) From B"), QIcon(QPixmap(iconB)), i18nc("Text used for select B when toolbar button.", "Choose\nB"), QKeySequence(Qt::CTRL | Qt::Key_2), this, [this]{choose(e_SrcSelector::B);}, ac, "merge_choose_b");
+    chooseC = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Select Line(s) From C"), QIcon(QPixmap(iconC)), i18nc("Text used for select C toolbar button.", "Choose\nC"), QKeySequence(Qt::CTRL | Qt::Key_3), this, [this]{choose(e_SrcSelector::C);}, ac, "merge_choose_c");
     autoAdvance = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Automatically Go to Next Unsolved Conflict After Source Selection"), QIcon(QPixmap(autoadvance)), i18nc("Auto goto next unsolved toolbar text.", "Auto\nNext"), this, &KDiff3App::slotAutoAdvanceToggled, ac, "merge_autoadvance");
 
     showWhiteSpaceCharacters = GuiUtils::createAction<KToggleAction>(i18n("Show Space && Tabulator Characters"), QIcon(QPixmap(showwhitespacechars)), i18nc("Show whitespace toolbar text.", "White\nCharacters"), this, &KDiff3App::slotShowWhiteSpaceToggled, ac, "diff_show_whitespace_characters");
@@ -703,9 +828,9 @@ void KDiff3App::initActions(KActionCollection* ac)
     showLineNumbers = GuiUtils::createAction<KToggleAction>(i18n("Show Line Numbers"), QIcon(QPixmap(showlinenumbers)), i18nc("Show line numbers toolbar text", "Line\nNumbers"), this, &KDiff3App::slotShowLineNumbersToggled, ac, "diff_showlinenumbers");
 
     mAutoSolve = GuiUtils::createAction<QAction>(i18n("Automatically Solve Simple Conflicts"), this, &KDiff3App::slotAutoSolve, ac, "merge_autosolve");
-    mUnsolve = GuiUtils::createAction<QAction>(i18n("Set Deltas to Conflicts"), this, &KDiff3App::slotUnsolve, ac, "merge_autounsolve");
-    mergeRegExp = GuiUtils::createAction<QAction>(i18n("Run Regular Expression Auto Merge"), this, &KDiff3App::slotRegExpAutoMerge, ac, "merge_regexp_automerge");
-    mMergeHistory = GuiUtils::createAction<QAction>(i18n("Automatically Solve History Conflicts"), this, &KDiff3App::slotMergeHistory, ac, "merge_versioncontrol_history");
+    mUnsolve = GuiUtils::createAction<QAction>(i18n("Set Deltas to Conflicts"), this, [this]{ Q_EMIT unsolve(); }, ac, "merge_autounsolve");
+    mergeRegExp = GuiUtils::createAction<QAction>(i18n("Run Regular Expression Auto Merge"), this, [this]{ Q_EMIT regExpAutoMerge(); }, ac, "merge_regexp_automerge");
+    mMergeHistory = GuiUtils::createAction<QAction>(i18n("Automatically Solve History Conflicts"), this, [this]{ Q_EMIT mergeHistory(); }, ac, "merge_versioncontrol_history");
     splitDiff = GuiUtils::createAction<QAction>(i18n("Split Diff At Selection"), this, &KDiff3App::slotSplitDiff, ac, "merge_splitdiff");
     joinDiffs = GuiUtils::createAction<QAction>(i18n("Join Selected Diffs"), this, &KDiff3App::slotJoinDiffs, ac, "merge_joindiffs");
 
@@ -736,7 +861,7 @@ void KDiff3App::initActions(KActionCollection* ac)
 
     mEscapeAction = new QShortcut(Qt::Key_Escape, this, this, &KDiff3App::slotFileQuit);
 
-    MergeResultWindow::initActions(ac);
+    m_pMergeResultWindow->initActions(ac);
 }
 
 void KDiff3App::showPopupMenu(const QPoint& point)
@@ -881,8 +1006,7 @@ void KDiff3App::slotFileSaveAs()
 
 void KDiff3App::slotFilePrint()
 {
-    if(m_pDiffTextWindow1 == nullptr || m_pDiffTextWindow2 == nullptr)
-        return;
+    assert(m_pDiffTextWindow1 == nullptr || m_pDiffTextWindow2 == nullptr);
 #ifdef QT_NO_PRINTER
     slotStatusMsg(i18n("Printing not implemented."));
 #else
